/***************************************************************************
                          astrodate.cpp  -  description
                             -------------------
    begin                : Fri Sep 15 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astrodate.h"
#include "astroutils.h"
#include "astroresources.h"
#undef i18n

#ifdef KDE
#include <kglobal.h>
#include <klocale.h>
#include <qdatetime.h>
#endif

extern "C"
{
# include <stdlib.h>
# include "swephexp.h"
};

double AstroDate::MdyToJulian()
{
  int greg = TRUE;
  if (Year < yeaJ2G || (Year == yeaJ2G &&
    (Month < monJ2G || (Month == monJ2G && Day < 15))))
    greg = FALSE;
  return floor(swe_julday(Year, Month, Day, 12.0, greg)+ROUND);
}

void AstroDate::JulianToMdy(double jd)
{
  int greg = jd < J2G ? FALSE : TRUE;
  double hour;
  swe_revjul(jd, greg, &Year, &Month, &Day, &hour);
}

double AstroDate::GetJulday()
{
  double x = Hm;
  x += DegToDec(-Offset);
  x -= Daylight ? 1.0 : 0.0;
  return MdyToJulian() + x / 24.0 - 0.5;
}

void AstroDate::SetJd(double jd)
{
  Julday = jd;
  jd += 0.5;
  double x = (Daylight ? -1.0 : 0.0) + DegToDec(-Offset), fra = jd - floor(jd), it = jd - fra;
  Hm = (fra * 24.0) - x;
  JulianToMdy(it);
  if(Corrected)
    Recompute();
}

void AstroDate::Now()
{
  struct tm tm, *tmptr;
  double lt;
  time_t t;
  int h, m;
  (void)time(&t);
  tmptr = gmtime(&t);
  tm = *tmptr;
  lt = RHM(tm.tm_hour, tm.tm_min);
  Aif->SetTZ(ZoneInfoFile);
  Aif->SetTZTime(lt, tm.tm_mday, tm.tm_mon + 1, tm.tm_year + 1900);
  Daylight = Aif->Isdst();
  Offset   = Aif->TimeOffset();
  lt       += Asr->ExtraTime + Offset;
  Day      = tm.tm_mday;
  Month    = tm.tm_mon + 1;
  Year     = tm.tm_year + 1900;
  Hm       = DegToDec(lt);
  Julday   = GetJulday();
 }

AstroDate::AstroDate(AstroInterface* aif)
{
  Aif = aif;
  Corrected = FALSE;
  SetTZ(Asr->DefZFile, FALSE);
  Daylight = FALSE;
  Offset = 0.0;
}

void AstroDate::SetTZ(const char* c, bool set)
{
  strncpy(ZoneInfoFile, c, BUFSIZE);
  if(set)
    Aif->SetTZ(ZoneInfoFile);
}

 AstroDate::AstroDate(int day, int month, int year, int hour, int minute, AstroInterface* aif)
{
  Aif = aif;
  Corrected = FALSE;
  SetTZ(Asr->DefZFile, FALSE);
  Daylight = FALSE;
  Offset = 0.0;
  SetDate(day, month, year, hour, minute);
}

AstroDate::AstroDate(int day, int month, int year, int hour, int minute, const char* zonefile, double zoneoffset = 0.0, bool dl = FALSE, AstroInterface* aif)
{
  Aif = aif;
  Corrected = FALSE;
  Day    = day;
  Month  = month;
  Year   = year;
  Set(hour, minute);
  Aif->SetTZ(zonefile ? zonefile : Asr->DefZFile);
  Aif->SetTZTime(RHM(hour, minute), Day, Month, Year);
  Offset = zonefile ? Aif->TimeOffset() : zoneoffset;
  Daylight = zonefile ? Aif->Isdst(): dl;
  Julday = GetJulday();
}

/** Sets the current date with the given parameters */
bool AstroDate::SetDate(int day, int month, int year, int hour, int minute)
{
  Day    = day;
  Month  = month;
  Year   = year;
  Set(hour, minute);
  return Recompute();
}

/** Recompute the julian day form the Day, Mont, etc... class members */
bool AstroDate::Recompute()
{
  if(Corrected)
  {
    Aif->SetTZ(ZoneInfoFile);
    Aif->SetTZTime(GetTime(), Day, Month, Year);
    Daylight = Aif->Isdst();
    Offset = Aif->TimeOffset();
  }
  Julday = GetJulday();
  return TRUE;
}

const AstroString& AstroDate::DayName() const
{
 return *Asr->DayNames[swe_day_of_week(Julday)];
}

const AstroString& AstroDate::DayShortName() const
{
  return *Asr->DayShortNames[swe_day_of_week(Julday)];
}

const AstroString& AstroDate::GetDate(AstroString& as) const
{
#ifdef KDE
const QDate qd(Year, Month, Day);
as = KGlobal::locale()->formatDate(qd, TRUE);
#endif
return as;
}

