/***************************************************************************
                          astrodate.h  -  description
                             -------------------
    begin                : Sat Sep 16 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTRODATE_H
#define ASTRODATE_H

#include "astrodefs.h"
#include "astrohm.h"
#include "astrointerface.h"

extern "C"
{
  //# include "sweodef.h"
# include <time.h>
};

/**
* The AstroDate is the standard kastrolog date and time management class.
* It provides a day, month, year <-> julian day conversion, a "right now"
* setting, a time increasing and decreasing
* @version $Id: 0.1
* @author Christophe Gros
*/

class AstroDate: public AstroHM
{
  public:

  /** Constructor */
  inline AstroDate() { Aif = 0; }

  /** Destructor */
  inline virtual ~AstroDate() {}

/** The default consructor. Sets the current time for now */
   AstroDate(AstroInterface *aif = Asf);

/** Yields a new date with usual parameters */
   AstroDate(int day, int month, int year, int hour, int minute, AstroInterface* aif = Asf);

/**
* Same as above, plus the zone info arguments
* @param zonefile The zone info file (e.g. "Europe/Brussels")
* @param zoneoffset The time zone offset
* @param daylight If daylight is active
* If zonefile is null, the zoneoffset and daylight are used instead
*/
   AstroDate(int day, int month, int year, int hour, int minute, const char* zonefile, double zoneoffset = 0.0, bool daylight = FALSE, AstroInterface* aif = Asf);

   /** Sets the current computing interface */
   inline void SetInterface(AstroInterface* aif) { Aif = aif; }

   /** Returns the current computing interface */
   inline AstroInterface* GetInterface() const { return Aif; }

/** Sets the current date with the given parameters */
  bool SetDate(int day, int month, int year, int hour, int minute);

/** Sets the time for now */
  void Now();

  /** Sets the timezone file name
      @param c the timezone filename
      @param set also sets the tzfile in the current computing interface
  */
  void SetTZ(const char* c, bool set = TRUE);

  /** Returns the name of the current day */
  const AstroString& DayName() const;

  /** Returns the short name of the current day */
  const AstroString& DayShortName() const;

/** Recomputes the julian day form the Day, Month, etc... class members */
  virtual bool Recompute();

/** Gets the current date + time */
  virtual inline void GetDate(int *day, int *month, int *year, int *hour, int* minute) const
  	{ HMR(Get(), *hour, *minute); *day = Day; *month = Month; *year = Year; }

/** Returns the current date according to the current locales
	@param as the date is also returned in as */
	virtual const AstroString& GetDate(AstroString& as) const;
	
  /** Returns the current julian day */
  virtual inline double GetJd() const { return Julday; }

  /** Returns the current time
      @param decimal if the return is decimal or minute */
  virtual inline double GetTime(bool decimal = FALSE) const { return Get(decimal); }

  /** Returns the current timezone hour.minute offset */
  virtual inline double GetOffset() const { return Offset; }

  /** Tells whether the daylight time saving is effective or not */
  virtual inline bool GetDaylight() const { return Daylight; }

  /** Sets the timezone offset and daylight */ 
  virtual inline void SetOffDay(double offset, bool daylight) { Offset = offset; Daylight = daylight; }

  /** Sets the julian day + Day, Month, Year setup + Daylight recomputation
      Corrected */
  void SetJd(double jd);

  bool Corrected;

 protected:
  
  AstroInterface* Aif;
  int Day, Month, Year;
  double Julday, Offset;
  bool Daylight;
    /** the corresponding zone info file */
  char ZoneInfoFile[BUFSIZE];

private:

  double MdyToJulian(), GetJulday();
  void JulianToMdy(double jd);
};

#endif
