/***************************************************************************
                          astrographics.h  -  description
                             -------------------
    begin                : Sat Dec 30 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROGRAPHICS_H
#define ASTROGRAPHICS_H

#include "../config.h"
#include "astrodefs.h"
#include "astrosigns.h"
#include "astrochartbase.h"

#ifdef KDE
#include <qwidget.h>
#include <qcolor.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qpen.h>
class QFont;
class QPicture;
#endif

/** A portable pixmap class */
class AstroPixmap
#ifdef KDE
: public QPixmap
#endif
{
 public:

  /** Constructor */
  inline AstroPixmap() {}

  /** Constructor */
  AstroPixmap(const char *c, const char* path = Asr->IconsPath);

  /** Constructor */
  inline AstroPixmap(int w, int h)
#ifdef KDE
    :QPixmap(w, h)
#endif
    {}

  /** Sets the pixmap with the file c */
  inline void SetPixmap(const char* c)
  {
#ifdef KDE
    load(c);
#endif
  }
};

/** This class implements a standard color management */
class AstroColor
#ifdef KDE
: public QColor
#endif
{
 public:
  
  inline AstroColor() {}
  inline AstroColor(int r, int g, int b)
# ifdef KDE
  : QColor(r, g, b)
# endif
    {}
  inline AstroColor(const char *name)
# ifdef KDE
  : QColor(name)
# endif
    {}
  inline void SetRgb(int r, int g, int b)
  {
# ifdef KDE
    setRgb(r, g, b);
#endif
  }
};

/** This class implements the standard 16 colors palette for all the drawings */
class AstroColorList
{
 public:

  AstroColorList();
  AstroColorList(unsigned const char*);
  ~AstroColorList();

  inline const AstroColor& operator[](int i) const { return Ac[i]; }

  void SetColor(AstroColor& ac, int i);

  void SetColor(unsigned char red, unsigned char green, unsigned char blue, int i);

  // The foreground and background AstroColor pointers
  AstroColor *Fore, *Back, *Grey;

  const AstroPixmap& operator % (int i) const { return *ColorPixmaps[i]; }

 private:

 AstroColor Ac[16];
 AstroPixmap *ColorPixmaps[16];
};


/** This class implements the management of a standard graphic field, with basic graphic primitives, such as lines. circles, etc..
*/

class AstroGraphics
{
 public:

  enum SizeChars { Very_Tiny, Tiny, Medium, Large, Huge };
  enum Justif    { Left, Right, Center };

  /** Constructor */
  AstroGraphics(AstroGraphics* parent = 0, int x = 0, int y = 0, int w = 200, int h = 200);

  /** Destructor */
  virtual ~AstroGraphics() {}

/** Moves the cursor to x, y */ 
  void Move(int x, int y);

/** Draws a line up to x, y */
 void Line(int x, int y);

 /** Draws a little dot */
 void Dot();

/** Draw a circle.
 @param x,y circle center
 @param r radius
 */
 void Circle(int x, int y, int r);

/** Sets current color */
 void Color(const AstroColor&);

/** Foreground color */
 inline void Fore() { Color(*Acl->Fore); }

 /** Background color */
 inline void Back() { Color(*Acl->Back); }

 /** Medium grey color */
 inline void Grey() { Color(*Acl->Grey); }

/** Sets the background Color */
 void Background(const AstroColor&);

 /** Gives the dash degree of a line. 0 = solid, 4 = fully dotted */
 void Dash(int d);

/** Sets the relative size of the text or astro font. This size is relative to
    the actual display size */
 void FontSize(enum SizeChars, bool isastro = FALSE);

/** Draws a line of unicode string */
 void Text(const AstroString& as, bool movex = FALSE);

 /** Draws a text with a given angle */
 void TextAngle(AstroString& a, double angle);

/** Draws a justified lie of text
 @param as the justified string to print
 @param justif Left, Right or Center
 @param len, the length of the string in pixels
*/
 void JustifText(const AstroString& as, enum Justif justif, int len);

/** Draws a sign glyph */
 void Sign(enum Signs);

/** Draws an astrofont glyph */
 void Glyph(char c);

/** Draws a justified glyph */
 void JustifGlyph(char c, enum Justif justif, int len);

/** Gives the AstroString metrics, (i.e. its width and height */
 void TextMetrics(const AstroString&, int* width, int* height);
 
/** Gives the astro glyph metrics, (i.e. its width and height) */
 void GlyphMetrics(char c, int* width, int* height);

 /** Draws a degree sign (glyph or name) min string */
 void DegSignMin(double d, bool name = FALSE, bool retro = FALSE);

 /** Draws a hh:mm:ss hour:minute:second string */
 void Hms(double d, const AstroString& sep, bool second = FALSE); 

 /** Returns drawing field width */
 virtual inline int GetWidth()  const { return Width; }

 /** Returns drwing field height */
 virtual inline int GetHeight() const { return Height; }

 /** Returns the Size factor */
 inline int GetSize() const { return Size; }

 protected:

 AstroGraphics* Base;

#ifdef KDE
 QPainter *Qp; 
 QPen *Pen;
 QFont *astro, *text;
#endif
 int Width, Height, PosX, PosY, Size, X, Y;
 void SetWidth(int i);
 void SetHeight(int i);

 private:

 
 char charbuf[2]; 
};

/** The effective field where all the graphic charts will be drawn  */
class GraphicChartBase : public AstroChartBase
{
 public:
 
  GraphicChartBase(ChartBaseParams *cbp, ObjsVector* ov, const AstroGraphics *ag) : AstroChartBase(cbp, ov) { Ag = (AstroGraphics *)ag; }

 inline void SetGraphicBase(AstroGraphics *ag) { Ag = ag; }

 /** abstract virtual methods which return the drawing space width and height */

 protected:

 AstroGraphics* Ag;
};

#endif
