/***************************************************************************
                          astroobjs.cpp  -  description
                             -------------------
    begin                : Sat Sep 23 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "astroobjs.h"
#include "astrochart.h"
#include "astrosigns.h"
#include "astroresources.h"
#include "astrographics.h"
#include "astrointerface.h"

extern "C"
{
#include "sweph.h"
};

AstroObjs::AstroObjs(AstroInterface* aif) : AstroData(aif), ChartParams(*Asr)
{
  Ac = 0;
  Asc = (*Asl)[Single];
  PlaChars = PLACHARS;
  Show = TRUE;
  DefSweFlags = 0;
  Ar = Drs;
}

AstroObjs::AstroObjs(AstroInterface *aif, AstroChart *ac) : AstroData(aif), ChartParams(*ac->Cbp)
{
  Ac = ac;
  Asc = (*Asl)[Single];
  PlaChars = PLACHARS;
  Show = TRUE;
  DefSweFlags = 0;
  Ar = Drs;
  RestNb = 0;
}

AstroObjs::AstroObjs(const AstroData& ad, AstroChart* ac, AstroSubChart* asc) : AstroData(ad), ChartParams(*ac->Cbp)
{
  Ac = ac;
  Asc = asc;
  PlaChars = PLACHARS;
  Show = TRUE;
  Ar = Drs;
  RestNb = 0;
  DefSweFlags = 0;
  Recompute();
}

AstroObjs::~AstroObjs(){
}

/** Returns the string value of the objet index, i.e. "12Cap45" */
bool AstroObjs::GetValStr(int index, AstroString& as) const
{
  double a;
  int s, deg, min;
  as = "";
  if(index < Sun || index > Last_Star || !(*Ar == index))
  {
    as = *Asr->ObjError;
    return FALSE;
  }
  a = Vals.Get(index);
  s = (int)a / 30;
  a -= (double)(s * 30);
  HMR(a, deg, min);
  if(deg < 10)
    as = " ";
  as += deg;
  as += (*Asg < s);
  a = (double)min * 60.0 / 100.0 + ROUND;
  min = (int)a;
  if(min < 10)
    as += "0";
  as += min;
  as += Vals.Retro(index) ? " R" : "  ";
#ifndef KDE
  as += '\0';
#endif
  return TRUE;
}

/** Returns the object name */
const AstroString& AstroObjs::operator << (int i) const
{
  if(i < 0 || i > Last_Star)
    return *Asr->ObjError;
  if(i > Last_Planet && i < First_Exotic)
    return *Asr->ObjError;
  if(i > Last_Exotic && i < Ascendant)
    return *Asr->ObjError;
  if(i >= First_Star && i <= Last_Star)
    return *Asr->StarsNames[i - First_Star];
  if(i >= First_Exotic && i <= Last_Exotic)
    i = i - First_Exotic + 23;
  if(i >= Ascendant && i <= House_12)
    i = i - Ascendant + 31;
  return *Asr->ObjNames[i];
}

/** Returns the short name of the corresponding object */
const AstroString& AstroObjs::operator < (int i) const
{
  if(i < 0 || i > Last_Star)
    return *Asr->ObjError;
  if(i > Last_Planet && i < First_Exotic)
    return *Asr->ObjError;
  if(i > Last_Exotic && i < Ascendant)
    return *Asr->ObjError;
  if(i >= First_Star && i <= Last_Star)
    return *Asr->StarsShortNames[i - First_Star];
  if(i >= First_Exotic && i <= Last_Exotic)
    i = i - First_Exotic + 23;
  if(i >= Ascendant && i <= House_12)
    i = i - Ascendant + 31;
  return *Asr->ObjShortNames[i];
}

/** Returns the reference to a static AstroString
containing the text value (a.k.a "12Cap45")
of the given object index */
const AstroString& AstroObjs::operator % (int index) const
{
  static AstroString as;
  (void)GetValStr(index, as);
  return as;
}

enum Elements AstroObjs::GetObjElement(int index) const
{
  if(index >= Sun && index <= Last_Planet)
    return Asr->ElemPla[index];
  if(index >= Ascendant && index <= House_12)
    return Asr->ElemHouses[index - Ascendant];
  return Undefined_Element;
}

const AstroColor& AstroObjs::GetObjColor(int index) const
{
  if(index >= Chiron && index <= Vesta)
    return COL[Asr->AsteroColor];
  if(index >= First_Exotic && index <= Last_Exotic)
    return COL[Asr->ExoticColor];
  switch(GetObjElement(index))
  {
    case Fire  : return COL[Asr->FireColor];
    case Air   : return COL[Asr->AirColor];
    case Water : return COL[Asr->WaterColor];
    case Earth : return COL[Asr->EarthColor];
    default    : return COL[Asr->OtherColor];
  }
}

char AstroObjs::GetObjChar(int index) const
{
  if(index > strlen(PlaChars))
    return 0;
  return PlaChars[index] != ' ' ? PlaChars[index] : 0;
}

/** Recompute the object longitudes according to the restrict array */
/*bool AstroObjs::DefaultRecompute()
{
  if(!AstroData::Recompute())
    return FALSE;
  return Aif->Recompute(&Vals, GetJd(), Latitude.Get(TRUE), Longitude.Get(TRUE), DefSweFlags, TRUE, RestNb);
}*/

void AstroObjs::CountFAWE(int* fire, int* air, int* water, int* earth) const
{
  int i;
  enum Elements e;
  *fire = *air = *water = *earth = 0;
  for(i = Sun; i < Last_Exotic; i++)
    switch(e = GetObjElement(i))
      {
        case Fire : (*fire)++;  break;
        case Air  : (*air)++;   break;
        case Water: (*water)++; break;
        case Earth: (*earth)++; break;
      }
}

void AstroObjs::CountCfm(int *cardinal, int *fixed, int *mutab) const
{
  int i;
  enum Signs s;
  *cardinal = *fixed = *mutab = 0;
  for(i = Sun; i < Last_Exotic; i++)
    if(*this == i)
      switch(s = Asg->GetSign(*this * i))
      {
	case Aries:
	case Cancer:
        case Libra:
	case Capricorn:  (*cardinal)++; break;
	case Taurus:
	case Leo:
	case Scorpio:
	case Aquarius:   (*fixed)++; break;
	case Gemini:
	case Virgo:
	case Sagittarius:
	case Pisces:     (*mutab)++;
      }
}

int AstroObjs::InHouse(int i, double *f = 0) const
{
  int j;
  if(!(*this == i)) return 0;
  double x = *this * i, x1, x2, x3;
  for(j = 0; j < 12; j++)
  {
    x1 = *this * (j + House_1);
    x2 = *this * (j + House_1 + 1);
    if(x2 < x1)
    {
      x2 += 360.0;
      x3 = x + 360.0;
      if(x3 >= x1 && x3 <= x2)
	x = x3;
    }
    if(x >= x1 && x <= x2)
    {
      if(f)
	 *f = 1000.0 * (x - x1) / (x2 - x1);
      return j + 1;
    }
  }
  return 0;
}

void AstroObjs::PutInHouse(int index, int house, double f)
{
  if(!(*this == index)) return;
  double x1, x2, x3;
  x1 = *this * (house + House_1 - 1);
  x2 = *this * (house + House_1);
  if(x2 < x1)
  {
    x2 += 360.0;
  }
  f = Normalize(x1 + (f * (x2 - x1) / 1000.0));
  Vals.CSet(index, f, Vals.Retro(index));
}

void AstroObjs::CountAsc(int *angular, int *succeedent, int *cadent) const
{
  int i, j;
  enum Signs s;
  *angular = *succeedent = *cadent = 0;
  for(i = Sun; i < Last_Exotic; i++)
    if(*this == i)
      switch(j = InHouse(i, 0))
      {
        case 1:
        case 4:
        case 7:
        case 10: (*angular)++; break;
        case 2:
        case 5:
        case 8:
        case 11: (*succeedent)++; break;
        case 3:
        case 6:
        case 9:
        case 12: (*cadent)++; break;
      }
}

bool AstroSubChart::Recompute(AstroObjs& ao, AstroChart& ac)
{
  if(!ao.AstroData::Recompute())
      return FALSE;
  return ao.AscRecompute(ao.GetJd()); 
}

class SingleSubChart: public AstroSubChart
{
public:

  inline SingleSubChart() : AstroSubChart(Single) {}
};

class TransitSubChart: public AstroSubChart
{
public:

  inline TransitSubChart() : AstroSubChart(Transit) {}
  bool Recompute(AstroObjs& ao, AstroChart& ac)
  {
    bool b, ih = ao.IfHouses();
    (~ao)->RestHouses(FALSE);
    b = AstroSubChart::Recompute(ao, ac);
    (~ao)->RestHouses(ih);
    return b;
  }
};

class DirectionSubChart: public AstroSubChart
{
public:

  inline DirectionSubChart() : AstroSubChart(Direction) {}
  bool Recompute(AstroObjs& ao, AstroChart& ac)
  {
    int i, f = ac.Acb->First(TRUE), y, mo, d, h, m, house[Undefined_Obj];
    if(f == -1 || !ao.IfHouses()) return FALSE;
    const AstroObjs& first = *ac[f];
    double jd1 = first.GetJd(), pos[Undefined_Obj];
    for(i = Sun; i <= Last_Planet; i++)
      if(first == i)
	house[i] = first.InHouse(i, &pos[i]);
    for(i = Ascendant; i <= House_12; i++)
      if(first == i)
	house[i] = first.InHouse(i, &pos[i]);
    if(first.IfExotic())
      for(i = First_Exotic; i <= Last_Exotic; i++)
	if(first == i)
	  house[i] = first.InHouse(i, &pos[i]);
    if(first.IfStars())
      for(i = First_Star; i <= Last_Star; i++)
	if(first == i)
	  house[i] = first.InHouse(i, &pos[i]);
    Diffs(ao.GetJd(), jd1, d, h, m);
    y  = (int)(d / 365.25);
    d -= (int)(y * 365.25);
    mo = (int)(d / 30.4375);
    d -= (int)(mo * 30.4375);
    double pjd = Direct(jd1, y, mo);
    if(!ao.AscRecompute(pjd)) return FALSE;
    for(i = Sun; i <= Last_Planet; i++)
      if(first == i)
	ao.PutInHouse(i, house[i], pos[i]);
    for(i = Ascendant; i <= House_12; i++)
      if(first == i)
	ao.PutInHouse(i, house[i], pos[i]);
    if(first.IfExotic())
      for(i = First_Exotic; i <= Last_Exotic; i++)
	if(first == i)
	  ao.PutInHouse(i, house[i], pos[i]);
    if(first.IfStars())
      for(i = First_Star; i <= Last_Star; i++)
	if(first == i)
	  ao.PutInHouse(i, house[i], pos[i]);
  }
  
};

class ProgressionSubChart: public AstroSubChart
{
public:

  inline ProgressionSubChart() : AstroSubChart(Progression) {}
  bool Recompute(AstroObjs& ao, AstroChart& ac)
  {
    int f = ac.Acb->First(TRUE), y, mo, d, h, m;
    const AstroObjs& a1 = *ac[f];
    if(f == -1) return FALSE;
    double jd1 = a1.GetJd();
    Diffs(ao.GetJd(), jd1, d, h, m);
    printf("d = %d\n", d);
    y = (int)(d / 365.25);
    d -= (int)(y * 365.25);
    mo = (int)(d / 30.4375);
    d -= (int)(mo * 30.4375);
    double pjd = Progress(jd1, y, mo, d);
    return ao.AscRecompute(pjd);
  }
};

AstroSubChartList::AstroSubChartList()
{
  Append(new AstroSubChart(NoSt));
  Append(new AstroSubChart(Show_Hide));
  Append(new SingleSubChart());
  Append(new TransitSubChart());
  Append(new DirectionSubChart());
  Append(new ProgressionSubChart());
}
