/***************************************************************************
                          astroobjs.h  -  description
                             -------------------
    begin                : Sat Sep 23 2000
    copyright            : (C) 2000 by Christophe Gros
    email                : skylendar@yahoo.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef ASTROOBJS_H
#define ASTROOBJS_H

#include "astrodata.h"
#include "astrorestrictions.h"
#include "astrointerface.h"

class AstroColor;

extern "C"
{
# include "swephexp.h"
};

class AstroObjs;
class AstroChart;

class AstroSubChart
{
 public:

  AstroSubChart( enum ChartSubType cst) { Cst = cst; }
  virtual const AstroString& GetName() const { return *Asr->SubTypesNames[Cst]; }
  virtual bool Recompute(AstroObjs& ao, AstroChart&);

  enum ChartSubType Cst;

 protected:

  inline Values& GetValues(const AstroObjs& ao) const;
};

class AstroSubChartList: public AstroList(AstroSubChart)
{
 public:

  AstroSubChartList();
  inline const AstroString& operator << (int i) { return (*this)[i]->GetName(); }
};

/**This class is the basic class which can, from the AstroData data,
compute the objects (planets, stars, etc...) parameters
  *@author Christophe Gros
  */

class AstroObjs : public AstroData, public ChartParams
{
public:

  friend class AstroSubChart;

  /** Constructor */
  AstroObjs(AstroInterface *aif = 0);

  /** Constructor */
  AstroObjs(AstroInterface *aif, AstroChart *ac);

  /** Constructor */
  AstroObjs(const AstroData& ad, AstroChart* ac, AstroSubChart* asc = (*Asl)[Single]);
  
  /** Destructor */
  virtual ~AstroObjs();

  // If the AstroObjs is shown or not
  bool Show;

  /** These methods and ops are simply the ones from Ad */ 
  inline bool operator==(int i) const { return *Ar == i; }
  inline bool IfHouses()        const { return Ar->IfHouses(); }
  inline bool IfExotic()        const { return Ar->IfExotic(); }
  inline bool IfStars()         const { return Ar->IfStars(); }
  inline bool IfMinor()         const { return Ar->IfMinor(); }
  inline int CountObjs()        const { return Ar->CountObjs(); }

/** Gets the ecliptical longitude of the object index */
  inline double operator * (int index) const { return Vals.Get(index); }

  /** Returns the 6 values array for th ith object */ 
  inline const DataValues* operator[](int index) const
    { return *Ar == index ? &Vals[index] : 0; }
  
  inline void SetSubChart(AstroSubChart* asc) { Asc = asc; }
  inline const AstroSubChart& GetSubChart() const { return *Asc; }
  inline enum ChartSubType GetSubType() const { return Asc->Cst; }

  /** Sets the restrictions group */
  inline void SetRestrict(AstroRestrictions* ar, int i)
    { Ar = ar; RestNb = i; }

  /** Returns a constant pointer to current AstroRestrictions */
  inline AstroRestrictions* operator ~() const { return Ar; }
  	
  /** Returns the string value of the objet index, i.e. "12Cap45 R"
  @param index, the object's index
  @param as The returned string
  @return true if ok
  */
 bool GetValStr(int index, AstroString& as) const;

  /** Returns the reference to a static AstroString
containing the text value (a.k.a "12Cap45")
of the given object index */
 const AstroString& operator % (int index) const;

 /** Returns the object name */
 const AstroString& operator << (int index) const;

 /** Returns the short name of the corresponding object */
 const AstroString& operator < (int index) const;

  /** Recomputes the objects longitude according to the restriction array */
 inline bool Recompute() { return Asc->Recompute(*this, *Ac); }

 inline bool AscRecompute(double jd) { return Aif->Recompute(&Vals, jd, Latitude.Get(TRUE), Longitude.Get(TRUE), DefSweFlags, TRUE, RestNb); }

  /** Returns the given object's element */
  enum Elements GetObjElement(int index) const;

  /** Returns the object's color */
  const AstroColor& GetObjColor(int index) const;

  /** Returns the character for the corresponding object glyph, otherwise 0 */
  char GetObjChar(int index) const;

  /** Counts the number of objects in fire/air/water/earth triplicity */
  void CountFAWE(int* fire, int* air, int* water, int* earth) const;

  /**Counts the number of objects in cardinal/fixed/mutable quadruplicity */
  void CountCfm(int *cardinal, int *fixed, int *mutab) const;

  /**Counts the number of objects in angular/succeedent/cadent quadruplicity */
  void CountAsc(int *angular, int *succeedent, int *cadent) const;

  /** Returns the number of the house where the object index dwells in */
  int InHouse(int index, double *f = 0) const;

  void PutInHouse(int index, int house, double f);

private:

  AstroChart* Ac;
  AstroSubChart* Asc;
  AstroRestrictions* Ar;
  int RestNb;
// The Zodiac longitudes array
  Values Vals;

  // The pointer of characters table for the astrofont glyphs	
  char *PlaChars;

  int DefSweFlags;
};

inline Values& AstroSubChart::GetValues(const AstroObjs& ao) const
{
  return (Values &)ao.Vals;
}
#endif

